//                      /,,
//                     /\  )
//                       \,'
//                    /`'`\
//
//                         ,
//                        ,@,
//                       ,@@@,
//                      ,@@@@@,
//               `@@@@@@@@@@@@@@@@@@@`
//                 `@@@@@@@@@@@@@@@`
//                   `@@@@@@@@@@@`
//                  ,@@@@@@`@@@@@@,
//                  @@@@`     `@@@@
//                 ;@`           `@;
//                   _   _   _   _
//                  (   (   (   |_)
//                   ~   ~   ~  |

uniform mat4 u_MVPMatrix;		// A constant representing the combined model/view/projection matrix.
uniform vec4 u_EyePos;
uniform vec4 u_SunConst;
uniform vec4 u_PlanetOffset;

attribute vec3 a_Position;		// Per-vertex position information we will pass in.
attribute vec3 a_Normal;		// Per-vertex normal information we will pass in.
attribute vec2 a_TexCoordinate; // Per-vertex texture coordinate information we will pass in.
attribute vec3 a_Tangent;		// Per-vertex tangent information we will pass in.
attribute vec3 a_Binormal;		// Per-vertex binormal information we will pass in.

varying vec4 v_Diffuse;
varying vec2 v_Spec;
varying vec3 v_Light;
varying vec3 v_View;


float Diffuse(vec3 Normal, vec4 lDir)
{
    float CosTheta;
    CosTheta = max(0.0, dot(Normal, lDir.xyz));

    return (CosTheta);
}


void main()                                                 	
{
    vec4 vPosOut = vec4(a_Position.xyz, 1.0);


	vec4 		lhtDir = vec4(10.0, 0.0, 10.0, 1.0);    //light Direction

	lhtDir.x = 25.0 * u_SunConst.x;
	lhtDir.z = 25.0 * u_SunConst.y;

	vec4		LightDir = normalize(lhtDir);

	vec3		vSunPosition = 2.0 * LightDir.xyz;

	vPosOut.xyz = vPosOut.xyz * u_SunConst.zzz;

	vec3 vNewNorm = normalize(a_Normal.xyz + 0.25 * LightDir.xyz);


	v_Light.xyz = vPosOut.xyz;
	v_View.xyz = u_EyePos.xyz;


	// diffuse
	float fbTmp = clamp(1.4 * Diffuse(vNewNorm, LightDir), 0.0, 1.0);
	v_Diffuse.b = fbTmp;
	v_Diffuse.g = pow(fbTmp / 1.4, 0.2);
	float frTmp = pow(fbTmp, 0.9);
	v_Diffuse.r = frTmp;
	v_Diffuse.a = max(frTmp, 0.05);
	v_Diffuse.b = mix(0.4, 1.0, frTmp);

	// ослепление солнцем
	vec3 vView	= normalize(vPosOut.xyz - u_EyePos.xyz);
	vec3 vToLight = normalize(lhtDir.xyz - vPosOut.xyz);
	float xRes = pow(clamp(dot(vToLight, vView), 0.0, 1.0), u_SunConst.w);
	v_Spec.x = 1.5 * xRes * clamp(1.0 - pow(clamp(dot(a_Normal, -LightDir.xyz), 0.0, 1.0), 0.5), 0.0, 1.0);
	v_Spec.y = 0.0;

	vPosOut.xyz =  vPosOut.xyz + u_PlanetOffset.xyz;

	gl_Position = u_MVPMatrix * vPosOut;
}                                                          