uniform mat4 u_MVPMatrix;		// A constant representing the combined model/view/projection matrix.
uniform vec4 u_EyePos;
uniform vec4 u_SunConst;
uniform vec4 u_PlanetOffset;

attribute vec3 a_Position;		// Per-vertex position information we will pass in.
attribute vec3 a_Normal;		// Per-vertex normal information we will pass in.
attribute vec2 a_TexCoordinate; // Per-vertex texture coordinate information we will pass in.
attribute vec3 a_Tangent;		// Per-vertex tangent information we will pass in.
attribute vec3 a_Binormal;		// Per-vertex binormal information we will pass in.

varying vec4 v_Diffuse;
varying vec2 v_Tex;
varying vec2 v_Spec;
varying vec3 v_Light;
varying vec3 v_View;


float Diffuse(vec3 Normal, vec3 lDir)
{
    float CosTheta;
    CosTheta = max(0.0, dot(Normal, lDir));

    return (CosTheta);
}

highp mat3 transposeMat3(highp mat3 inMatrix) {
    highp vec3 i0 = inMatrix[0];
    highp vec3 i1 = inMatrix[1];
    highp vec3 i2 = inMatrix[2];

    highp mat3 outMatrix = mat3(
                 vec3(i0.x, i1.x, i2.x),
                 vec3(i0.y, i1.y, i2.y),
                 vec3(i0.z, i1.z, i2.z)
                 );

    return outMatrix;
}

void main()                                                 	
{
    vec4 vPosOut = vec4(a_Position.xyz + u_PlanetOffset.xyz, 1.0);

    vec3		LightDir;
	LightDir.x = u_SunConst.x;
	LightDir.y = 0.0;
	LightDir.z = u_SunConst.y;

	LightDir = normalize(LightDir);

	vec3		vPos = vPosOut.xyz;

	vec3 vNewNorm = normalize(a_Normal.xyz + 0.35 * LightDir.xyz);

	mat3 matTangentSpace;

	matTangentSpace[0] = a_Tangent;  // x
    matTangentSpace[1] = a_Binormal; // y
    matTangentSpace[2] = a_Normal;  // z

    matTangentSpace = transposeMat3(matTangentSpace);


	vec3 vView = vPos.xyz - u_EyePos.xyz;

	vView = normalize(vView);

	// Calculate light vector and move into tangent space
    v_Light = matTangentSpace * LightDir;
    // Calculate view vector and move into tangent space
    v_View = matTangentSpace * vView;

  	// diffuse
	float tmpB = clamp(1.4 * Diffuse(vNewNorm, LightDir), 0.0, 1.0);
	v_Diffuse.b = tmpB;
	v_Diffuse.g = pow(tmpB / 1.4, 0.1);
	v_Diffuse.r = pow(tmpB, 0.9);
	v_Diffuse.a = u_PlanetOffset.w;


	// specular
	vec3 relfectedVector = normalize(reflect(vView, a_Normal));
    //calculate the sunlight reflection
	float fSpecDot = clamp(dot(relfectedVector,LightDir), 0.0, 1.0);
	float tmpDiff = Diffuse(a_Normal, LightDir);
	v_Spec.y = 1.8*pow(fSpecDot,6.0) * tmpDiff;

	v_Spec.x = pow(clamp(dot(-vView, a_Normal), 0.0, 1.0), 0.5);

    v_Tex = a_TexCoordinate;

	gl_Position = u_MVPMatrix * vPosOut;
}
