precision highp float;       	// Set the default precision to medium. We don't need as high of a
								// precision in the fragment shader.

uniform sampler2D u_TextureMap;    // The input Map texture.
uniform sampler2D u_TextureBump;   // The input Bump texture.

uniform float u_Brightness;
  
varying vec4 v_Diffuse;
varying vec2 v_Tex;
varying vec2 v_Spec;
varying vec3 v_Light;
varying vec3 v_View;

// The entry point for our fragment shader.
void main()                    		
{
    vec3 sunlightColor;
	sunlightColor.r = 250.0/255.0;
	sunlightColor.g = 227.0/255.0;
	sunlightColor.b = 226.0/255.0;


	// sample all 4 textures
	vec4 vCol0 = texture2D(u_TextureMap, v_Tex); // day colormap with specularmap
	vec4 vCol2 = texture2D(u_TextureBump, v_Tex); // normalmap

	vec2 vTex1;
	vTex1.y = v_Tex.y;

	vTex1.x = v_Tex.x + (v_Diffuse.a * 4.0); // cloud offset
    vec4 vCol3 = texture2D(u_TextureMap, vTex1); // clouds colormap

    vTex1.x = v_Tex.x + (v_Diffuse.a * 4.7); // cloud offset
    vec4 vCol4 = texture2D(u_TextureMap, vTex1); // clouds colormap

    vTex1.x = v_Tex.x + (v_Diffuse.a * 5.5); // cloud offset
    vec4 vCol5 = texture2D(u_TextureMap, vTex1); // clouds colormap

	vTex1.x = v_Tex.x + (v_Diffuse.a * 6.4); // cloud offset
	vec4 vCol6 = texture2D(u_TextureMap, vTex1); // clouds colormap


    vec3 vNorm = vCol2.xyz;

	// unpack normals
	vNorm = (vNorm * 2.0) - 1.0;
	vNorm.xyz = normalize(vNorm.xyz);

	// bump
	float bumpScaler = mix(1.0, 1.2, v_Diffuse.r);
	vec4 n_dot_l = vec4(clamp(bumpScaler * dot(vNorm,v_Light), 0.0, 1.0));
	vec4 diffuse = max(n_dot_l,0.15);

	// calc colormap between day colormap and night colormap and apply lighting to day colormap
	vec4 finalColor = vCol0 * diffuse;

	vCol3.rgb = vCol3.rgb * diffuse.rgb;
	vCol4.rgb = vCol4.rgb * diffuse.rgb;
	vCol5.rgb = vCol5.rgb * diffuse.rgb;
	vCol6.rgb = vCol6.rgb * diffuse.rgb;

	vCol3.a = vCol3.a * 0.37;
	vCol4.a = vCol4.a * 0.37;
	vCol5.a = vCol5.a * 0.37;
	vCol6.a = vCol6.a * 0.37;

	// add clouds shadow
	finalColor.rgb = mix(finalColor.rgb, vCol3.rgb, vCol3.a);
	finalColor.rgb = mix(finalColor.rgb, vCol4.rgb, vCol4.a);
	finalColor.rgb = mix(finalColor.rgb, vCol5.rgb, vCol5.a);
	finalColor.rgb = mix(finalColor.rgb, vCol6.rgb, vCol6.a);

	finalColor.rgb = clamp(finalColor.rgb * u_Brightness, 0.0, 1.0);

	finalColor.a = 1.0;

    gl_FragColor = finalColor;
}

