precision highp float;       	// Set the default precision to medium. We don't need as high of a
								// precision in the fragment shader.

uniform sampler2D u_TextureMap;    // The input Map texture.
uniform sampler2D u_TextureBump;   // The input Bump texture.
uniform sampler2D u_TextureCloud;   // The input Cloud texture.

uniform float u_Brightness;
  
varying vec4 v_Diffuse;
varying vec2 v_Tex;
varying vec2 v_Spec;
varying vec3 v_Light;
varying vec3 v_View;
varying vec3 v_Par;
  
// The entry point for our fragment shader.
void main()                    		
{
    vec3 sunlightColor;
	sunlightColor.r = 250.0/255.0;
	sunlightColor.g = 227.0/255.0;
	sunlightColor.b = 226.0/255.0;

	// for parallax
	float scalePar = 0.002;
	float bPar = scalePar * 0.5;
	vec4 heightPar = scalePar * texture2D(u_TextureBump, v_Tex) - bPar;
	vec2 Nt;
	Nt.y = v_Tex.y - (-v_Par.y)*heightPar.a;
	Nt.x = v_Tex.x + (v_Par.x)*heightPar.a;

	// sample all 4 textures
	vec4 vCol0 = texture2D(u_TextureMap, Nt); // day colormap with specularmap
	vec4 vCol1 = texture2D(u_TextureBump, Nt); // Nt // normalmap

	vec2 vTex1;
	vTex1.y = v_Tex.y;

	vTex1.x = v_Tex.x + (v_Diffuse.a * 3.9); // cloud offset
    vec4 vCol3 = texture2D(u_TextureCloud, vTex1); // clouds colormap

    vTex1.x = v_Tex.x + (v_Diffuse.a * 4.7); // cloud offset
    vec4 vCol4 = texture2D(u_TextureCloud, vTex1); // clouds colormap

    vTex1.x = v_Tex.x + (v_Diffuse.a * 7.3); // cloud offset
    vec4 vCol5 = texture2D(u_TextureCloud, vTex1); // clouds colormap


    vec3 vNorm = vCol1.xyz;

	// unpack normals
	vNorm = (vNorm * 2.0) - 1.0;
	vNorm.xyz = normalize(vNorm.xyz);

	// bump
	vec4 n_dot_l = vec4(clamp(1.2 * dot(vNorm,v_Light), 0.0, 1.0));
	vec4 diffuse = max(n_dot_l,0.15/1.1);
//	diffuse = lerp(diffuse * 0.5f, n_dot_l, Inp.vDiffuse.r);

	// calc colormap between day colormap and night colormap and apply lighting to day colormap
	vec4 finalColor = clamp(1.1 * vCol0 * diffuse, 0.0, 1.0);

	// add specular color using specularmap in alpha channel of day colormap
//	finalColor.rgb = finalColor.rgb + Inp.vSpec.y * sunlightColor;

    // calculate clouds shadow using clouds texture
/*    vCol3.rgb = vCol3.rgb * v_Diffuse.r;
    vCol4.rgb = vCol4.rgb * v_Diffuse.r;
    vCol5.rgb = vCol5.rgb * v_Diffuse.r;*/

	float nightCloud = clamp(diffuse.x * 0.35 + v_Diffuse.r * 0.65, 0.0, 1.0);
	float diffCloud = mix(nightCloud, v_Diffuse.r, v_Diffuse.r);

	vCol3.rgb = vCol3.rgb * diffCloud;
	vCol4.rgb = vCol4.rgb * diffCloud;
	vCol5.rgb = vCol5.rgb * diffCloud;

    float alphaScaler = clamp(v_Diffuse.r * 2.0, 0.0, 1.0);

    vCol3.a = mix(vCol3.a, vCol3.a * 1.7, alphaScaler);
    vCol4.a = mix(vCol4.a, vCol4.a * 1.7, alphaScaler);
    vCol5.a = mix(vCol5.a, vCol5.a * 1.7, alphaScaler);

    // add clouds shadow
    finalColor.rgb = mix(finalColor.rgb, vCol3.rgb, vCol3.a);
    finalColor.rgb = mix(finalColor.rgb, vCol4.rgb, vCol4.a);
    finalColor.rgb = mix(finalColor.rgb, vCol5.rgb, vCol5.a);

	finalColor.rgb = clamp(finalColor.rgb * u_Brightness, 0.0, 1.0);

	finalColor.a = 1.0;

    gl_FragColor = finalColor;
}

